# Docker control
import json
import os

import docker
from sltxpkg.globals import DOCKER_URL

# TODO for now we need a reading PAT?


class DockerCtrl:
    def __init__(self):
        self.client = docker.from_env()
        self.client.ping()

    def update_img(self, profile: str):
        target = DOCKER_URL.format(**locals())
        print("Pulling image:", target, "this may take a few minutes")
        for line in self.client.api.pull(target, tag='latest', stream=True):
            # default values
            d = {'status': 'unknown', 'progress': '', 'id': ''}
            d = {**d, **json.loads(line)}
            print("   {status} {progress} {id}".format(**d))

    def run_in_container(self, profile: str, command: str):
        if profile.startswith(":"):
            target = profile[1:]
        else:
            target = DOCKER_URL.format(**locals())
        print("Launching container",target)
        run = self.client.containers.run(
            target, command=command, detach=True, remove=True, working_dir='/home',
            volumes={
                os.getcwd(): {
                    'bind': '/home',
                    'mount': 'rw'
                }
            })
        for l in run.logs(stdout=True, stderr=True, stream=True, timestamps=True):
            print(l.decode('utf-8'), end='')
        print("Container completed.")


import sltxpkg.prompt as prompt
from sltxpkg.lithie.docker_mg import DockerCtrl
import sltxpkg.globals as sg

# TODO: get valid names
def install():
    valid_profiles=['tx-small', 'tx-default', 'tx-full']
    print("Please enter the profile you want. Valid names are:", valid_profiles)
    target = prompt.get("Profile [{default}]", default=sg.configuration[sg.C_DOCKER_PROFILE]).lower()
    # TODO: do not exit, try anyway
    if target not in valid_profiles:
        print(target, "was not found in", valid_profiles, "exiting for now")
        exit(1)
    docker_ctrl = DockerCtrl()
    docker_ctrl.update_img(target)

def compile():
    docker_ctrl = DockerCtrl()
    profile = sg.configuration[sg.C_DOCKER_PROFILE] if sg.args.profile is None else sg.args.profile
    sltx_command = "sltx -t " + str(sg.args.threads) + " "

    if sg.args.config is not None:
        sltx_command += "--config \"" + sg.args.config + "\" "
    
    sltx_command += "raw-compile "

    if sg.args.recipe is not None:
        sltx_command += "--recipe \"" + sg.args.recipe + "\" "

    sltx_command += "\"" + sg.args.file + "\""

    print("Running command in docker: " + sltx_command)
    docker_ctrl.run_in_container(profile, sltx_command)